SIGNALR = "v2.0/signalr"
SIGNALR_METHOD = "Broadcast"
SIGNALR_METHOD_ERROR = "Error"
SIGNALR_HUB_FC = "BroadcastHubV2"

FC_GET_OTP                          = "api/v2/Trading/GetOTP"
FC_GET_ACCESS_TOKEN                 = "api/v2/Trading/AccessToken"

FC_NEW_ORDER                        = "api/v2/Trading/NewOrder"
FC_MODIFY_ORDER                     = "api/v2/Trading/ModifyOrder"
FC_CANCEL_ORDER                     = "api/v2/Trading/CancelOrder"

FC_DER_NEW_ORDER                    = "api/v2/Trading/derNewOrder"
FC_DER_MODIFY_ORDER                 = "api/v2/Trading/derModifyOrder"
FC_DER_CANCEL_ORDER                 = "api/v2/Trading/derCancelOrder"

FC_GET_ORDER_HISTORY                = "api/v2/Trading/orderHistory"
FC_GET_ORDER_BOOK                   = "api/v2/Trading/orderBook"
FC_GET_AUDIT_ORDER_BOOK             = "api/v2/Trading/auditOrderBook"

FC_GET_DER_POSITION                 = "api/v2/Trading/derivPosition"
FC_GET_STOCK_POSITION               = "api/v2/Trading/stockPosition"
FC_GET_MAX_BUY_QUANTITY             = "api/v2/Trading/maxBuyQty"
FC_GET_MAX_SELL_QUANTITY            = "api/v2/Trading/maxSellQty"

FC_GET_ACCOUNT_BALANCE              = "api/v2/Trading/cashAcctBal"
FC_GET_DER_ACCOUNT_BALANCE          = "api/v2/Trading/derivAcctBal"
FC_GET_PP_MMR_ACCOUNT               = "api/v2/Trading/ppmmraccount"

FC_GET_RATELIMIT                    = "api/v2/Trading/rateLimit"

#CASH API
FC_CASH_CIA_AMOUNT                  = "api/v2/cash/cashInAdvanceAmount"
FC_CASH_UNSETTLE_SOLD_TRANSACTION   = "api/v2/cash/unsettleSoldTransaction"
FC_CASH_TRANSFER_HISTORY            = "api/v2/cash/transferHistories"
FC_CASH_CIA_HISTORY                 = "api/v2/cash/cashInAdvanceHistories"
FC_CASH_CIA_EST_FEE                 = "api/v2/cash/estCashInAdvanceFee"
FC_CASH_VSD_DW                      = "api/v2/cash/vsdCashDW"
FC_CASH_TRANSFER                    = "api/v2/cash/transferInternal"
FC_CASH_CIA                         = "api/v2/cash/createCashInAdvance"

#ONLINE RIGHT SUBSCRIPTION
FC_ORS_DIVIDEND                     = "api/v2/ors/dividend"
FC_ORS_EXCERCISABLE_QTY             = "api/v2/ors/exercisableQuantity"
FC_ORS_HISTORY                      = "api/v2/ors/histories"
FC_ORS                              = "api/v2/ors/create"

#STOCK
FC_STOCK_TRANSFERABLE               = "api/v2/stock/transferable"
FC_STOCK_HISTORY                    = "api/v2/stock/transferHistories"
FC_STOCK_TRANSFER                   = "api/v2/stock/transfer"

